==> tuxurf-0.02.tar.gz: HTW/url.c <==
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <fcntl.h>
#include <ghttp.h>

#include "parser.h"

static ghttp_request *http_request;
static char *base_url;
static char *base_host;
static char *base_path;

static int is_full_url(char *p)
{
	char *colon = strchr(p,':');
	char *slash = strchr(p, '/');
	if(colon && (slash==NULL || colon < slash))
		return 1;
	return 0;
}

static void set_url_pointers(void)
{
	char *p=strchr(base_url,':');
	base_host = ++p;
	while(*p=='/')
		p++;
	p=strchr(p, '/');
	if(p)
		base_path = p;
	else
		base_path = base_url+strlen(base_url);
}

void set_base_url(char *p)
{
	char *segment;
	
	if(base_url)
		set_url_pointers();
	
	if(is_full_url(p))
	{
		if(base_url)
			free(base_url);
		base_url = strdup(p);
		return;
	}
	if(*p=='/')
	{
		/* Host relative */
		char *buf=malloc(strlen(base_url)-strlen(base_path)+strlen(p)+2);
		*base_path=0; /* The '/' */
		sprintf(buf, "%s%s", base_url, p);
		if(base_url)
			free(base_url);
		base_url = buf;
		return;
	}
	segment = strrchr(base_path, '/');
	if(segment)
		*segment = 0;
	{
		char *buf=malloc(strlen(base_url)+strlen(p)+2);
		sprintf(buf, "%s/%s", base_url, p);
		if(base_url)
			free(base_url);
		base_url = buf;
	}
}	


static void load_error(const char *why, char *url)
{
	/* FIXME: Quote URL */
	snprintf(buf, sizeof(buf),
	"<HTML><HEAD><TITLE>Error</TITLE></HEAD>"
	"<BODY><H3>%s</H3>"
	"<P>Unable to fetch '%s'.</BODY></HTML>", why, url);
	pagebuf=buf;
	mark_html_dirty();
}

void load_url(void)
{
	/* Hack loader */
	if(http_request)
	{
		*buf=0;
		pagebuf=buf;
		ghttp_request_destroy(http_request);	
		link_free();
	}
	reset_scrollbar();
	http_request = ghttp_request_new();
	if(ghttp_set_uri(http_request, base_url)!=0)
	{
		load_error("Unsupported or invalid URL", base_url);
		return;
	}

	if(ghttp_prepare(http_request)!=0)
	{
		load_error("Unable to make connection", base_url);
		return;
	}

	switch(ghttp_process(http_request))
	{
		default:
		case ghttp_not_done:
			load_error("Unable to fetch the page\n", base_url);
			return;
		case ghttp_done:
			pagebuf = ghttp_get_body(http_request);
			break;
		case ghttp_error:
			load_error(ghttp_get_error(http_request), base_url);
			return;
	}
	/* HACK!! */
	pagebuf[ghttp_get_body_len(http_request)-1]=0;
	mark_html_dirty();
}

==> tuxurf-0.03.tar.gz: HTW/url.c <==
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <fcntl.h>
#include <ghttp.h>

#include "parser.h"

static ghttp_request *http_request;
static char *base_url;
static char *base_host;
static char *base_path;

static int is_full_url(char *p)
{
	char *colon = strchr(p,':');
	char *slash = strchr(p, '/');
	if(colon && (slash==NULL || colon < slash))
		return 1;
	return 0;
}

static void set_url_pointers(void)
{
	char *p=strchr(base_url,':');
	base_host = ++p;
	while(*p=='/')
		p++;
	p=strchr(p, '/');
	if(p)
		base_path = p;
	else
		base_path = base_url+strlen(base_url);
}

char *url_anchor(char *p)
{
	/* Given a URL figure out the anchor */
	char *x=p+strlen(p)-1;
	while(x > p)
	{
		/* Beware the entity monster */
		if(*x=='#' && x[-1]!='&')
		{
			*x=0;
			return x+1;
		}
		x--;
	}
	if(*p=='#')
	{
		*p=-0;
		return p+1;
	}
	return NULL;
}		

void set_base_url(char *p)
{
	char *segment;
	
	if(base_url)
		set_url_pointers();
	
	if(is_full_url(p))
	{
		if(base_url)
			free(base_url);
		base_url = strdup(p);
		return;
	}
	if(*p=='/')
	{
		/* Host relative */
		char *buf=malloc(strlen(base_url)-strlen(base_path)+strlen(p)+2);
		*base_path=0; /* The '/' */
		sprintf(buf, "%s%s", base_url, p);
		if(base_url)
			free(base_url);
		base_url = buf;
		return;
	}
	if(*p=='#')
	{
		/* Anchor */
		char *buf=malloc(strlen(base_url)+strlen(p)+1);
		sprintf(buf, "%s%s", base_url, p);
		if(base_url)
			free(base_url);
		base_url = buf;
		return;
	}		
	segment = strrchr(base_path, '/');
	if(segment)
		*segment = 0;
	{
		char *buf=malloc(strlen(base_url)+strlen(p)+2);
		sprintf(buf, "%s/%s", base_url, p);
		if(base_url)
			free(base_url);
		base_url = buf;
	}
}	


static void load_error(const char *why, char *url)
{
	/* FIXME: Quote URL */
	snprintf(buf, sizeof(buf),
	"<HTML><HEAD><TITLE>Error</TITLE></HEAD>"
	"<BODY><H3>%s</H3>"
	"<P>Unable to fetch '%s'.</BODY></HTML>", why, url);
	pagebuf=buf;
	mark_html_dirty();
}

void load_url(void)
{
	int s;
	int retries = 0;
	char *anchor;
	
retry:
	/* Hack loader */
	if(http_request)
	{
		*buf=0;
		pagebuf=buf;
		ghttp_request_destroy(http_request);	
		link_free();
	}
	
	/* Go to the top of the page */
	set_scrollbar(0);
	
	/* Split off the anchor */
	anchor = url_anchor(base_url);

	printf("Fetching %s\n", base_url);
		
	http_request = ghttp_request_new();
	if(ghttp_set_uri(http_request, base_url)!=0)
	{
		load_error("Unsupported or invalid URL", base_url);
		return;
	}

	if(ghttp_prepare(http_request)!=0)
	{
		load_error("Unable to make connection", base_url);
		return;
	}

	switch(ghttp_process(http_request))
	{
		default:
		case ghttp_not_done:
			load_error("Unable to fetch the page", base_url);
			return;
		case ghttp_done:
			pagebuf = ghttp_get_body(http_request);
			break;
		case ghttp_error:
			load_error(ghttp_get_error(http_request), base_url);
			return;
	}
	s = ghttp_status_code(http_request);
	
	printf("%d %s\n", s, ghttp_reason_phrase(http_request));
	
	switch(s)
	{
		case 200:	/* We got the page */
		case 201:	/* We should not see these */
		case 202:
		case 203:
		case 204:
		case 205:
		case 206:
			break;
		case 300:	/* TODO - multiple choices */
			break;
		case 301:	/* Its moved forever - update bookmarks etc */
		case 302:	/* Its moved temporarily */
		{
			char *p;
			const char *where = ghttp_get_header(http_request,"Location");
			if(where==NULL)
				break;
			retries++;
			if(retries == 10)
				load_error("Endlessly redirected", base_url);
			/*
			 *	Try the suggested new URL
			 */
			p = strdup(where);
			set_base_url(p);
			free(p);
			goto retry;			
		}
		case 303:	/* See other - FIXME */
		case 304:	/* Not modified - use when we have caching */
			break;
		case 305:	/* Do proxy setup eventually */
			break;
		default:;
	}
	/* HACK!! */
	pagebuf[ghttp_get_body_len(http_request)-1]=0;
	
	if(anchor)
		set_scrollbar(find_anchor(anchor));
	mark_html_dirty();
}
